--=============================================================================
library IEEE, std;
use IEEE.electrical_systems.all;
use IEEE.math_real.all;
use std.textio.all;
--=============================================================================

package StatEye_functions is

type vector_of_integers      is array(positive range <>) of integer;
type vector_of_reals         is array(positive range <>) of real;
type vector_of_real_pairs    is array(positive range <>) of vector_of_reals(1 to 2);
type vector_of_voltages      is array(positive range <>) of voltage;
type vector_of_voltage_pairs is array(positive range <>) of vector_of_voltages(1 to 2);

--=============================================================================
function FirstLastX (Cursors    : vector_of_reals;
                     Resolution : real;
                     Selector   : character) return real;
-------------------------------------------------------------------------------
function FindCursorIndex (Wfm : vector_of_reals) return integer;
-------------------------------------------------------------------------------
procedure Make3Dfile (CPDF           : vector_of_real_pairs;
                      IterationCount : integer;
                      Resolution     : real;
                      TimeStep       : real);
-------------------------------------------------------------------------------
function MakeCPDF (PDFx           : vector_of_reals;
                   PDFy           : vector_of_reals;
                   RoundedCursor  : real;
                   BERvalues      : vector_of_reals;
                   IterationCount : integer;
                   Resolution     : real;
                   TimeStep       : real;
                   WriteFile      : boolean := false) return vector_of_real_pairs;
-------------------------------------------------------------------------------
function Convolve (Cursors        : vector_of_reals;
                   BERvalues      : vector_of_reals;
                   IterationCount : integer;
                   Resolution     : real;
                   TimeStep       : real;
                   WriteFile      : boolean := false) return vector_of_real_pairs;
-------------------------------------------------------------------------------
function MakeEyeContours (Wfm            : vector_of_reals;
                          UnitInterval   : real;
                          CursorIndex    : integer;
                          BERvalues      : vector_of_reals;
                          IterationCount : integer;
                          Resolution     : real;
                          TimeStep       : real;
                          WriteFile      : boolean := false) return vector_of_real_pairs;
--=============================================================================

end package StatEye_functions;

--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

package body StatEye_functions is

--=============================================================================
function FirstLastX (Cursors    : vector_of_reals;
                     Resolution : real;
                     Selector   : character) return real is
-------------------------------------------------------------------------------
  variable ReturnVal : real := 0.0;
-------------------------------------------------------------------------------
begin
  -----------------------------------------------------------------------------
  -- This function returns the lowest or highest possible x-value when
  -- convolving the pre and post cursors in "Cursors" (i.e. not including
  -- the value in the last position).  Each pre and post cursor value is
  -- rounded up or down to increments of "Resolution" to get the most
  -- positive or most negative return value, i.e. the rounding error is
  -- cumulative in the result.
  -----------------------------------------------------------------------------
  if Selector = 'F' then

    for i in Cursors'left to Cursors'right-1 loop
      if Cursors(i) < 0.0 then
        ReturnVal := ReturnVal + Resolution*floor(Cursors(i)/Resolution);
      end if;
    end loop;

  elsif Selector = 'L' then

    for i in Cursors'left to Cursors'right-1 loop
      if Cursors(i) > 0.0 then
        ReturnVal := ReturnVal + Resolution*ceil(Cursors(i)/Resolution);
      end if;
    end loop;

  end if;

  return ReturnVal;
  -----------------------------------------------------------------------------
end function FirstLastX;
--=============================================================================
function FindCursorIndex (Wfm : vector_of_reals) return integer is
-------------------------------------------------------------------------------
  variable Index : integer := 0;
  variable Value : real    := Wfm(Wfm'left);
-------------------------------------------------------------------------------
begin
  -----------------------------------------------------------------------------
  -- This function finds the index of the peak value in Wfm.
  -----------------------------------------------------------------------------
  for i in Wfm'left+1 to Wfm'right loop
    if Wfm(i) > Value then
      Value := Wfm(i);
      Index := i;
    end if;
  end loop;
  -----------------------------------------------------------------------------
  return Index;
  -----------------------------------------------------------------------------
end function FindCursorIndex;
--=============================================================================
procedure Make3Dfile (CPDF           : vector_of_real_pairs;
                      IterationCount : integer;
                      Resolution     : real;
                      TimeStep       : real) is
-------------------------------------------------------------------------------
  file     FileObject : text open append_mode is "StatEyeData.m";
  variable Xline      : line;
  variable Yline      : line;
-------------------------------------------------------------------------------
begin
  -----------------------------------------------------------------------------
  -- This procedure opens a file (or creates one if it doesn't exist yet) in
  -- append mode and writes the data that was generated by MakeCPDF in it.
  -- The file name is hard coded in this example, but there is no technical
  -- limitation that would prevent it from being parameterized so that it
  -- could be passed down from higher levels.
  -----------------------------------------------------------------------------
  -- If this is running for the first time in the simulation, print the
  -- value of Resolution and TimeStep to the file.
  -----------------------------------------------------------------------------
  if IterationCount = 1 then
    write(Xline, string'("Resolution = " & real'image(Resolution) & ";"));
    writeline(FileObject, Xline);
    write(Xline, string'("TimeStep = " & real'image(TimeStep) & ";"));
    writeline(FileObject, Xline);
  end if;
  -----------------------------------------------------------------------------
  -- Write the variable name and its content for the DataX cell array.
  -----------------------------------------------------------------------------
  write(Xline, string'("DataX{" & integer'image(IterationCount) & ",1} = ["));
  writeline(FileObject, Xline);
  for i in CPDF'range loop
    write(Xline, CPDF(i)(1), right, 12, 8);
    writeline(FileObject, Xline);
  end loop;
  write(Xline, string'("];"));
  writeline(FileObject, Xline);
  -----------------------------------------------------------------------------
  -- Write the variable name and its content for the DataY cell array.
  -----------------------------------------------------------------------------
  write(Yline, string'("DataY{" & integer'image(IterationCount) & ",1} = ["));
  writeline(FileObject, Yline);
  for i in CPDF'range loop
    write(Yline, CPDF(i)(2), right, 12, 8);
    writeline(FileObject, Yline);
  end loop;
  write(Yline, string'("];"));
  writeline(FileObject, Yline);
  -----------------------------------------------------------------------------
end procedure Make3Dfile;
--=============================================================================
function MakeCPDF (PDFx           : vector_of_reals;
                   PDFy           : vector_of_reals;
                   RoundedCursor  : real;
                   BERvalues      : vector_of_reals;
                   IterationCount : integer;
                   Resolution     : real;
                   TimeStep       : real;
                   WriteFile      : boolean := false) return vector_of_real_pairs is
-------------------------------------------------------------------------------
  variable ContourVec : vector_of_real_pairs(1 to BERvalues'length) := (others => (others => 0.0));
  variable CPDF       : vector_of_real_pairs(1 to 2*PDFx'length-2)  := (others => (0.0, 0.5));

  constant ResolutionBy2  : real := Resolution/2.0;
  variable NextLeftPoint  : real := 0.5;
  variable NextRightPoint : real := 0.5;

  constant PDFxRightP1  : integer := PDFx'right+1;
  constant CPDFRightP1  : integer := CPDF'right+1;
  constant CPDFRightP2  : integer := CPDF'right+2;

  constant CPDFstartL   : integer := 1;
  constant CPDFendL     : integer := PDFx'right-1;
  constant CPDFstartR   : integer := PDFx'right;
  constant CPDFendR     : integer := CPDF'right;

  variable PDFleft_L2R  : integer := 2;
  variable PDFright_L2R : integer := 2;
  variable PDFright_R2L : integer := PDFx'right-1;
  variable PDFleft_R2L  : integer := PDFx'right-1;

  variable CPDF_i_L2R   : integer := 1;
  variable CPDF_i_R2L   : integer := CPDF'right;

  variable LeftX_L2R    : real := 0.0;
  variable RightX_L2R   : real := 0.0;
  variable RightX_R2L   : real := 0.0;
  variable LeftX_R2L    : real := 0.0;

  variable KeepGoing    : boolean := true;
-------------------------------------------------------------------------------
begin
  -----------------------------------------------------------------------------
  -- This function generates two cumulative probability distribution functions
  -- (CPDF) from the probability distribution function (PDF) of the inter
  -- symbol interference (ISI) only.  The first half (left) of CPDF contains
  -- the CPDF of a logic "0" and the second half (right) of CPDF contains the
  -- CPDF for a logic "1".
  --
  -- The function will also generate contour lines for the BER levels provided
  -- in BERvalues and it will pass a CPDF to Make3Dfile function to generate
  -- a data file that can be used by other tools for 3D plotting.
  --
  -- The function returns the contours to its caller so that they can be
  -- plotted by the simulator in 2D.
  -----------------------------------------------------------------------------
  -- Reverse integrate (by subtraction) from the outside going inward to
  -- generate the two CPDF functions.
  -----------------------------------------------------------------------------
  for k in 2 to PDFx'right-1 loop
    CPDF(k-1)(1)           := PDFx(k);
    CPDF(k)(2)             := CPDF(k-1)(2) - PDFy(k);
    CPDF(CPDFRightP2-k)(1) := PDFx(PDFxRightP1-k) + RoundedCursor;
    CPDF(CPDFRightP1-k)(2) := CPDF(CPDFRightP2-k)(2) - PDFy(PDFxRightP1-k);
  end loop;

  CPDF(CPDFendL)(1)   := PDFx(CPDFendL) + Resolution;
  CPDF(CPDFstartR)(1) := PDFx(2) + RoundedCursor - Resolution;

  -----------------------------------------------------------------------------
  -- Extract the contours from CPDF at the values given in BERvalues for 2D
  -- plotting.
  -----------------------------------------------------------------------------
  for k in BERvalues'range loop
    ---------------------------------------------------------------------------
    -- Lower eye contours.
    ---------------------------------------------------------------------------
    if (CPDF(CPDFstartL)(2) < BERvalues(k)) then
      ContourVec(k)(1) := CPDF(CPDFstartL)(1);
    else
      for m in CPDFendL-1 downto CPDFstartL loop
        if (CPDF(m)(2) >= BERvalues(k)) then
          ContourVec(k)(1) := CPDF(m)(1);
          exit;
        end if;
      end loop;
    end if;
    ---------------------------------------------------------------------------
    -- Upper eye contours.
    ---------------------------------------------------------------------------
    if (CPDF(CPDFendR)(2) < BERvalues(k)) then
      ContourVec(k)(2) := CPDF(CPDFendR)(1);
    else
      for m in CPDFstartR+1 to CPDFendR loop
        if (CPDF(m)(2) >= BERvalues(k)) then
          ContourVec(k)(2) := CPDF(m)(1);
          exit;
        end if;
      end loop;
    end if;
    ---------------------------------------------------------------------------
  end loop;
  -----------------------------------------------------------------------------

  if WriteFile then
    ---------------------------------------------------------------------------
    if (CPDF(CPDFstartR)(1) - CPDF(CPDFendL)(1)) >= ResolutionBy2 then
      -------------------------------------------------------------------------
      -- If the left CPDF ends without touching or ovelapping the right CPDF
      -- make a 3D file using the entire CPDF as it is now.
      -------------------------------------------------------------------------
      Make3Dfile(CPDF, IterationCount, Resolution, TimeStep);
    elsif abs(CPDF(CPDFstartR)(1) - CPDF(CPDFendL)(1)) < ResolutionBy2 then
      -------------------------------------------------------------------------
      -- If the left CPDF ends on the same point as the right CPDF starts
      -- make a 3D file skipping that point.
      -------------------------------------------------------------------------
      Make3Dfile(CPDF(1 to CPDFendL) & CPDF(CPDFstartR+1 to CPDFendR),
                 IterationCount,
                 Resolution,
                 TimeStep);
    else 
      -------------------------------------------------------------------------
      -- If there is an overlap between the left and right CPDFs, redo the
      -- integation taking this overlap into account, and then make a 3D file
      -- using that CPDF.  This will be done by going inward from the ends of
      -- the CPDF until reaching the middle.
      -------------------------------------------------------------------------
      if abs(RoundedCursor) < ResolutionBy2 then
        PDFleft_L2R  := 2;
        PDFright_L2R := 2;
        PDFright_R2L := PDFx'right;
        PDFleft_R2L  := PDFx'right-1;
      else
        PDFleft_L2R  := 2;
        PDFright_L2R := 1;
        PDFright_R2L := PDFx'right-1;
        PDFleft_R2L  := PDFx'right;
      end if;

      LeftX_L2R  := PDFx(PDFleft_L2R);
      RightX_L2R := PDFx(PDFright_L2R) + RoundedCursor;
      RightX_R2L := PDFx(PDFright_R2L) + RoundedCursor;
      LeftX_R2L  := PDFx(PDFleft_R2L);

      while KeepGoing loop
        -----------------------------------------------------------------------
        if (abs(LeftX_L2R - RightX_L2R) < ResolutionBy2) then
          ---------------------------------------------------------------------
          CPDF(CPDF_i_L2R)(1)   := PDFx(PDFright_L2R) + RoundedCursor;
          if (PDFright_L2R > 1) then
            CPDF(CPDF_i_L2R)(2) := NextLeftPoint + PDFy(PDFright_L2R) - PDFy(PDFleft_L2R);
            NextLeftPoint       := CPDF(CPDF_i_L2R)(2);
          else
            CPDF(CPDF_i_L2R)(2) := NextLeftPoint;
            NextLeftPoint       := CPDF(CPDF_i_L2R)(2) - PDFy(PDFleft_L2R);
          end if;
          PDFleft_L2R  := PDFleft_L2R + 1;
          PDFright_L2R := PDFright_L2R + 1;
          LeftX_L2R    := PDFx(PDFleft_L2R);
          RightX_L2R   := PDFx(PDFright_L2R) + RoundedCursor;
          ---------------------------------------------------------------------
        elsif (LeftX_L2R < RightX_L2R) then
          CPDF(CPDF_i_L2R)(1) := PDFx(PDFleft_L2R);
          CPDF(CPDF_i_L2R)(2) := NextLeftPoint;
          NextLeftPoint       := CPDF(CPDF_i_L2R)(2) - PDFy(PDFleft_L2R);
          PDFleft_L2R := PDFleft_L2R + 1;
          LeftX_L2R   := PDFx(PDFleft_L2R);
          ---------------------------------------------------------------------
        elsif (LeftX_L2R > RightX_L2R) then
          CPDF(CPDF_i_L2R)(1) := PDFx(PDFright_L2R) + RoundedCursor;
          CPDF(CPDF_i_L2R)(2) := NextLeftPoint + PDFy(PDFright_L2R);
          NextLeftPoint       := CPDF(CPDF_i_L2R)(2);
          PDFright_L2R := PDFright_L2R + 1;
          RightX_L2R   := PDFx(PDFright_L2R) + RoundedCursor;
          ---------------------------------------------------------------------
        end if;
        -----------------------------------------------------------------------
        if (abs(RightX_R2L - LeftX_R2L) < ResolutionBy2) then
          ---------------------------------------------------------------------
          CPDF(CPDF_i_R2L)(1)   := PDFx(PDFright_R2L) + RoundedCursor;
          if (PDFleft_R2L < PDFx'right) then
            CPDF(CPDF_i_R2L)(2) := NextRightPoint + PDFy(PDFleft_R2L) - PDFy(PDFright_R2L);
            NextRightPoint      := CPDF(CPDF_i_R2L)(2);
          else
            CPDF(CPDF_i_R2L)(2) := NextRightPoint;
            NextRightPoint      := CPDF(CPDF_i_R2L)(2) - PDFy(PDFright_R2L);
          end if;
          PDFright_R2L := PDFright_R2L - 1;
          PDFleft_R2L  := PDFleft_R2L - 1;
          RightX_R2L   := PDFx(PDFright_R2L) + RoundedCursor;
          LeftX_R2L    := PDFx(PDFleft_R2L);
          ---------------------------------------------------------------------
        elsif (RightX_R2L > LeftX_R2L) then
          CPDF(CPDF_i_R2L)(1) := PDFx(PDFright_R2L) + RoundedCursor;
          CPDF(CPDF_i_R2L)(2) := NextRightPoint;
          NextRightPoint      := CPDF(CPDF_i_R2L)(2) - PDFy(PDFright_R2L);
          PDFright_R2L := PDFright_R2L - 1;
          RightX_R2L   := PDFx(PDFright_R2L) + RoundedCursor;
          ---------------------------------------------------------------------
        elsif (RightX_R2L < LeftX_R2L) then
          CPDF(CPDF_i_R2L)(1) := PDFx(PDFleft_R2L);
          CPDF(CPDF_i_R2L)(2) := NextRightPoint + PDFy(PDFleft_R2L);
          NextRightPoint      := CPDF(CPDF_i_R2L)(2);
          PDFleft_R2L := PDFleft_R2L - 1;
          LeftX_R2L   := PDFx(PDFleft_R2L);
          ---------------------------------------------------------------------
        end if;
        -----------------------------------------------------------------------
        if ((CPDF(CPDF_i_R2L)(1) - CPDF(CPDF_i_L2R)(1)) < ResolutionBy2) or
           ((CPDF_i_R2L - CPDF_i_L2R) <= 1) then
          KeepGoing := false;
          if (CPDF(CPDF_i_R2L)(1) < CPDF(CPDF_i_L2R)(1)) then
            CPDF_i_L2R := CPDF_i_L2R - 1;
            CPDF_i_R2L := CPDF_i_R2L + 1;
          elsif (CPDF(CPDF_i_R2L)(1) = CPDF(CPDF_i_L2R)(1)) then
            CPDF(CPDF_i_L2R)(2) := realmax(CPDF(CPDF_i_L2R)(2), CPDF(CPDF_i_R2L)(2));
            CPDF_i_R2L := CPDF_i_R2L + 1;
          end if;
        else
          CPDF_i_L2R := CPDF_i_L2R + 1;
          CPDF_i_R2L := CPDF_i_R2L - 1;
        end if;
        -----------------------------------------------------------------------
      end loop;
      -------------------------------------------------------------------------
      -- Generate a 3D file with the new CPDF.
      -------------------------------------------------------------------------
      Make3Dfile(CPDF(1 to CPDF_i_L2R) & CPDF(CPDF_i_R2L to CPDF'right),
                 IterationCount,
                 Resolution,
                 TimeStep);
      -------------------------------------------------------------------------
    end if;
  end if;
  -----------------------------------------------------------------------------
  return ContourVec;
  -----------------------------------------------------------------------------
end function MakeCPDF;
--=============================================================================
function Convolve (Cursors        : vector_of_reals;
                   BERvalues      : vector_of_reals;
                   IterationCount : integer;
                   Resolution     : real;
                   TimeStep       : real;
                   WriteFile      : boolean := false) return vector_of_real_pairs is
-------------------------------------------------------------------------------
  constant Coefficient : real    := 0.5**(Cursors'length);
  constant FirstX      : real    := FirstLastX(Cursors, Resolution, 'F');
  constant LastX       : real    := FirstLastX(Cursors, Resolution, 'L');
  constant IndexOfZero : integer := integer(-FirstX/Resolution)+1;

  variable Yy      : vector_of_reals(1 to integer((LastX-FirstX)/Resolution)+1) := (others => 0.0);
  variable YyCopy  : vector_of_reals   (Yy'range) := (others => 0.0);
  variable Yptr    : vector_of_integers(Yy'range) := (others => 0);

  variable Xvec1   : vector_of_reals(1 to 1) := (others => 0.0);
  variable Xvec2   : vector_of_reals(1 to 1) := (others => 0.0);
  variable Yvec1   : vector_of_reals(1 to 1) := (others => 0.0);
  variable Yvec2   : vector_of_reals(1 to 1) := (others => 0.0);

  variable m       : integer := 0;
  variable YyIndex : integer := 0;
  variable LengthS : integer := 0;
-------------------------------------------------------------------------------
begin
  -----------------------------------------------------------------------------
  -- This function generates a probability distribution function (PDF) for
  -- the inter symbol interference (ISI) from the pre and post cursor values
  -- provided in "Cursors" by convolution.  Several assumptions are made, so
  -- this is not a general purpose convolution function.  The values in
  -- "Cursors" are expected to be arranged so that the last position holds the
  -- cursor and all other positions hold the pre and post cursors of the pulse
  -- response waveform.  Each of these values are assumed to have a 50-50%
  -- probability with a zero, therefore their hight is assumed to be 0.5.
  -- The cursor will be "convolved" with the PDF in the MakeCPDF function.
  -----------------------------------------------------------------------------
  -----------------------------------------------------------------------------
  -- Yy contains an integer count of the delta diracs in the location of the
  -- corresponding voltage along the Yx axis.
  -- Initialize Yptr so that it contains the index to the first pre/post cursor
  -- and zero pair from the list in "Cursors".
  -- Initialize Yy so that it contains a copy of where Yptr is pointing.
  -- This is the same as having done a convolution with a delta dirac at zero
  -- volts already.
  -- The voltages are converted so that their value falls within the interval
  -- of +/-(Resolution/2) around the Yx point.  The conversion to integer type
  -- takes care of this rounding automatically.
  -----------------------------------------------------------------------------
  Yptr(1)     := integer((Cursors(1)-FirstX)/Resolution)+1;
  Yptr(2)     := IndexOfZero;
  Yy(Yptr(1)) := Yy(Yptr(1)) + 1.0;
  Yy(Yptr(2)) := Yy(Yptr(2)) + 1.0;

  for i in 2 to Cursors'right-1 loop
    ---------------------------------------------------------------------------
    -- Find the non zero elements in Yy and copy their index values to the
    -- beginning of Yptr to be used in the convolution of Yy with new value
    -- coming from "Cursors".  A copy of Yy is made to YyCopy to avoid self
    -- distruction.
    ---------------------------------------------------------------------------
    m := 0;
    for j in Yy'range loop
      if (Yy(j) > 0.0) then
        m       := m+1;
        Yptr(m) := j;
      end if;
    end loop;
    LengthS := m;
    YyCopy  := Yy;
    ---------------------------------------------------------------------------
    -- This is the convolution loop.
    ---------------------------------------------------------------------------
    for k in 1 to LengthS loop
      YyIndex     := Yptr(k) + integer((Cursors(i)-FirstX)/Resolution)+1 - IndexOfZero;
      Yy(YyIndex) := Yy(YyIndex) + YyCopy(Yptr(k));
    end loop;
    ---------------------------------------------------------------------------
  end loop;
  -----------------------------------------------------------------------------
  -- Rearrange Yy eliminating the elements that have a zero in Yy, multiply
  -- the Yy values by "Coefficient" and generate a corresponding Yx vector
  -- (using YyCopy to save memory).
  -----------------------------------------------------------------------------
  m := 0;
  for j in Yy'range loop
    if (Yy(j) > 0.0) then
      m         := m+1;
      YyCopy(m) := FirstX + real(j-1)*Resolution;
      Yy(m)     := Yy(j) * Coefficient;
    end if;
  end loop;
  -----------------------------------------------------------------------------
  -- Generate a cumulative PDF (CPDF) function using MakeCPDF and extract eye
  -- contours for 2D plotting (also done by MakeCPDF).
  -- If WriteFile is true, MakeCPDF will call Make3Dfile to generate an ASCII
  -- data file for 3D plotting with Matlab (or other external tools).
  -----------------------------------------------------------------------------
  Xvec1(1) := YyCopy(1) - Resolution;
  Xvec2(1) := YyCopy(m) + Resolution;

  return MakeCPDF(Xvec1 & YyCopy(1 to m) & Xvec2,
                  Yvec2 & Yy(1 to m) & Yvec2,
                  Resolution*round(Cursors(Cursors'right)/Resolution),
                  BERvalues,
                  IterationCount,
                  Resolution,
                  TimeStep,
                  WriteFile);
  -----------------------------------------------------------------------------
end function Convolve;
--=============================================================================
function MakeEyeContours (Wfm            : vector_of_reals;
                          UnitInterval   : real;
                          CursorIndex    : integer;
                          BERvalues      : vector_of_reals;
                          IterationCount : integer;
                          Resolution     : real;
                          TimeStep       : real;
                          WriteFile      : boolean := false) return vector_of_real_pairs is
-------------------------------------------------------------------------------
  constant UIpts          : integer := integer(floor(UnitInterval/TimeStep));
  constant CursorWindow   : integer := ((CursorIndex-UIpts/2)/UIpts)+1;
  constant StartIndex     : integer :=   CursorIndex-UIpts/2 - (CursorWindow-1)*UIpts;
  constant NoOfWindows    : integer := (Wfm'length-StartIndex)/UIpts;

  variable NonCursors     : vector_of_reals(1 to NoOfWindows) := (others => 0.0);
  variable NonCursorCount : integer := 0;
-------------------------------------------------------------------------------
begin
  -----------------------------------------------------------------------------
  -- This function samples the pulse response in intervals of UIpts and
  -- passes all of the cursors to the Convolve function to perform the
  -- statistical eye analysis.  The BERvalues vector is also passed to
  -- Convolve so that the eye contours can be extracted for each element in
  -- BERvalues.  The eye contours returned by Convolve are passed back to the
  -- caller of this function and can eventually be plotted by the simulator.
  -----------------------------------------------------------------------------
  for i in NonCursors'range loop
    if i = CursorWindow then
      NonCursors(NonCursors'right) := Wfm(StartIndex + IterationCount-1 + (i-1)*UIpts);
    else
      NonCursorCount               := NonCursorCount + 1;
      NonCursors(NonCursorCount)   := Wfm(StartIndex + IterationCount-1 + (i-1)*UIpts);
    end if;
  end loop;
  -----------------------------------------------------------------------------
  return Convolve(NonCursors,
                  BERvalues,
                  IterationCount,
                  Resolution,
                  TimeStep,
                  WriteFile);
  -----------------------------------------------------------------------------
end function MakeEyeContours;
--=============================================================================

end package body StatEye_functions;

--=============================================================================
