%**************************************************************************
% This code reads the data file named StatEyeData.m and plots its content
% on a 3D surface plot.  The data consists of two cell arrays called
% DataX and DataY.  The corresponding cells of the DataX and DataY cell
% array must have vectors of the same length, but the vectors do not have
% to be the same length in each element of the cell array.  However, the
% values in the DataX vectors must fall on a grid that is common to all.
%**************************************************************************
% Load the raw data
%**************************************************************************
clc;
StatEyeData
%**************************************************************************
% Find the maximum voltage range of the data and
% create a voltage and time vector
%**************************************************************************
MinXarray = zeros(length(DataX),1);
MaxXarray = zeros(length(DataX),1);
for i=1:length(DataX)
  MinXarray(i) = DataX{i}(1);
  MaxXarray(i) = DataX{i}(end);
end
voltage = min(MinXarray) : Resolution : max(MaxXarray);
time    = TimeStep : TimeStep : TimeStep*length(DataX);
%**************************************************************************
% Create an animation of a BER vs. voltage bath tub curve for all times
%**************************************************************************
figure(1);
for i = 1:length(DataX)
  plot(DataX{i}, DataY{i}, '.-')
  grid
  title(['BER vs. voltage at time ', sprintf('%2.2e', time(i)), ' of the UI window']);
  xlabel('voltage');
  ylabel('BER');
  axis ([min(MinXarray) max(MaxXarray) 0 0.5]);
  pause(0.1)
end
%**************************************************************************
% Create a matrix (Z) and fill it with the data
%**************************************************************************
Z = zeros(length(time), length(voltage));
for i=1:length(time)
  for j=1:length(DataX{i})
    Z(i,find(abs(voltage-DataX{i}(j)) < Resolution/2)) = DataY{i}(j);
  end
end
%**************************************************************************
% Fill in all missing points in a sample and hold fashion to make a
% smooth surface.
% NOTE:  This loop can take a very long time to complete when the
%        resolution is set to smaller values in the simulation.
%        (Less than 1.0e-3 is not recommended.)  There are probably
%        better ways of doing this in Matlab...
%**************************************************************************
for i=1:size(Z,1)                      % loop along the time axis
  HoldLeft  = 0.5;
  HoldRight = 0.5;
  LeftIndex  = int32(1);
  RightIndex = int32(size(Z,2));

  while (RightIndex-LeftIndex) > 0     % loop along the voltage axis
    if Z(i,LeftIndex) == 0
      Z(i,LeftIndex) = HoldLeft;
    else
      HoldLeft = Z(i,LeftIndex);
    end
    if Z(i,RightIndex) == 0
      Z(i,RightIndex) = HoldRight;
    else
      HoldRight = Z(i,RightIndex);
    end
    LeftIndex  = LeftIndex+1;
    RightIndex = RightIndex-1;
  end % while loop

  % Fill the middle point in case we have an odd number of points
  if (LeftIndex == RightIndex)
    Z(i,LeftIndex) = max(Z(i,LeftIndex-1),Z(i,LeftIndex+1));
  end

end % for loop
%**************************************************************************
% Create a 3D plot
%**************************************************************************
figure(2);
surfc(voltage, time, Z);
title('BER vs. time and voltage');
xlabel('voltage');
ylabel('time');
zlabel('PDF');
%set(gca,'ZScale','log')
axis tight;
shading interp;
%shading faceted;
grid on;
view(55,45);
colorbar;
%**************************************************************************
% Create a contour plot at levels listed in BERvalues
%**************************************************************************
BERvalues = [0.0, 0.05, 0.1, 0.15, 0.2, 0.25, 0.3, 0.35, 0.4, 0.45, 0.5];
figure(3);
contourf(voltage, time, Z, BERvalues);
title('BER vs. time and voltage');
xlabel('voltage');
ylabel('time');
axis tight;
grid on;
view(90,-90);
colorbar;
%**************************************************************************